@@ -1,3 +1,5 @@
+# -*- coding: utf-8 -*-
+from __future__ import print_function, division, absolute_import
 from numba.control_flow.control_flow import (ControlBlock, ControlFlowAnalysis,
                                              FuncDefExprNode)
 from numba.control_flow.cfstats import *
