@@ -297,9 +297,9 @@ def create_dtypes():
         uint32   : np.uint32,
         uint64   : np.uint64,
 
-        float_   : np.float32,
-        double   : np.float64,
         longdouble: np.longdouble,
+        double   : np.float64,
+        float_   : np.float32,
 
         short    : np.dtype('h'),
         int_     : np.dtype('i'),
@@ -508,7 +508,7 @@ class Type(miniutils.ComparableObjectMixin):
         return getattr(type(self), attr)
 
     def __call__(self, *args):
-        """Return a FunctionType with return_type and args set
+        """Return a function with return_type and args set
         """
         if len(args) == 1 and not isinstance(args[0], Type):
             # Cast in Python space
