@@ -117,7 +117,8 @@ class SpecializeComparisons(visitors.NumbaTransformer):
 
             if left.type.is_array or right.type.is_array:
                 # array < x -> Array(bool_, array.ndim)
-                result_type = self.context.promote_types(left.type, right.type)
+                result_type = self.env.crnt.typesystem.promote(
+                    left.type, right.type)
             else:
                 result_type = bool_
 
