@@ -26,4 +26,4 @@ def convert_to_int(s, base):
 
 if __name__ == '__main__':
     import numba
-    numba.testmod()
+    numba.testing.testmod()
