@@ -13,9 +13,10 @@ import numba
 from numba import *
 import numpy as np
 
-@autojit
+@autojit(warnstyle="simple")
 def tuple_unpacking_error(obj):
     a, b = obj
+    print(a, b)
 
 if __name__ == "__main__":
     numba.testing.testmod()
